from __future__ import unicode_literals


try:
    from django.core.urlresolvers import reverse, NoReverseMatch
except ImportError:
    from django.urls import reverse, NoReverseMatch

from .encoder import JsonDocEncoder
from .decoder import JsonDocDecoder


class JsonBaseManager(object):
    def __init__(self, json_data):
        self.json_data = json_data

    def get_instance(self, data):
        return JsonDocDecoder(data).instance

    def _label(self, data):
        return data.get('label', None)

    def _url(self, data):
        # FIXME: typo handling for old releases
        kwargs = data['url'].get(
                'url_parametres',
                data['url'].get('url_parameters'))
        try:
            return reverse(data['url']['url_name'],
                           kwargs=kwargs)
        except NoReverseMatch:
            return None

    def short_description(self):
        raise NotImplementedError

    def url(self):
        raise NotImplementedError

    def description(self):
        raise NotImplementedError


class JsonObjectManager(JsonBaseManager):
    def url(self):
        return self._url(self.json_data)

    def short_description(self):
        return self._label(self.json_data)

    def description(self):
        return self._label(self.json_data)

    @property
    def instance(self):
        return self.get_instance(self.json_data)


class JsonListManager(JsonBaseManager):
    def __iter__(self):
        for item in self.json_data or []:
            yield self.get_instance(item)

    def __getitem__(self, key):
        if hasattr(self, key) and callable(getattr(self, key)):
            return getattr(self, key)

        obj = self.get_first(key)
        if obj:
            return self.get_instance(obj)
        return None

    def iter_data(self):
        for item in self.json_data or []:
            if item:
                yield item

    @property
    def instance(self):
        for item in self.iter_data():
            if item.get('model', ''):
                return self.get_instance(item)
        return None

    def get_first(self, key):
        for item in self.iter_data():
            if item.get('model', '') == key:
                return item
        return None

    def append(self, instance):
        self.json_data.append(JsonDocEncoder(instance).dump())

    def label(self):
        for item in self.iter_data():
            label = self._label(item)
            if label is not None:
                return label
        return None

    def url(self):
        for item in self.iter_data():
            if 'url' in item:
                return self._url(item)
        return None

    def description(self):
        rc = []
        for item in self.iter_data():
            label = self._label(item)
            if label:
                rc.append(label)
        return ", ".join(rc)

    def iter_items(self):
        for obj in self.iter_data():
            item = obj.copy()
            if 'url' in obj:
                url = self._url(obj)
                item['url'] = url
            item['label'] = self._label(obj)
            yield item

    def short_description(self):
        return self.description()

    def update(self, obj):
        """
        Replace object in the json_data with new provided. Keep ordering.
        :param obj: object to replace with
        """
        rc = []
        data = JsonDocEncoder(obj).dump()
        for item in self.iter_data():
            if item['model'] == data['model']:
                rc.append(obj)
            else:
                rc.append(self.get_instance(item))
        return rc


class JsonDocManager(JsonListManager):
    def __init__(self, *args, **kwargs):
        super(JsonDocManager, self).__init__(*args, **kwargs)

        raise DeprecationWarning("JsonDocManager deprecated, please use JsonListManager instead")



from __future__ import unicode_literals
from django.utils.encoding import force_text


class JsonDocEncoder(object):
    def __init__(self, data_object):
        self.data_object = data_object

    def key(self):
        return {
            'id': self.data_object.id,
            'module': self.data_object.__class__.__module__,
            'model': self.data_object.__class__.__name__}

    def dump(self):
        data = self.key()
        data['label'] = getattr(self.data_object, 'label', force_text(self.data_object))

        if hasattr(self.data_object, 'json_dump'):
            data.update(self.data_object.json_dump())

        return data



from __future__ import unicode_literals
from importlib import import_module


class JsonDocDecoder(object):
    def __init__(self, obj_key):
        self.obj_key = obj_key

    def __str__(self):
        return "%s.%s [%s]" % (self.obj_key['module'],
                               self.obj_key['model'],
                               self.obj_key['id'])

    @property
    def instance(self):
        _module = import_module(self.obj_key['module'])
        cls = getattr(_module, self.obj_key['model'])
        handler = getattr(cls, 'json_get_instance', None)
        if handler and callable(handler):
            return handler(data)
        return cls.objects.get(pk=self.obj_key['id'])



#!/usr/bin/env python
try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


Name='ua2.jsondoc'
ProjecUrl="https://bitbucket.org/sdh-llc/ua2.jsondoc"
Version='0.3.1'
Author='Viacheslav Vic Bukhantsov'
AuthorEmail='vic@ua2crm.com'
Maintainer='Viacheslav Vic Bukhantsov'
Summary='Library for serialize django model header to json dictionary'
License='BSD License'
ShortDescription=Summary

with open('README.rst') as fh:
    long_description = fh.read()

needed = [
]

EagerResources = [
    'ua2',
]

ProjectScripts = [
##    'scripts/runweb',
]

PackageData = {
    '': ['*.*', '*.html'],
}

# Make exe versions of the scripts:
EntryPoints = {
}

setup(
    url=ProjecUrl,
    name=Name,
    zip_safe=False,
    version=Version,
    author=Author,
    author_email=AuthorEmail,
    description=ShortDescription,
    long_description=long_description,
    license=License,
    scripts=ProjectScripts,
    install_requires=needed,
    include_package_data=True,
    packages=find_packages('src'),
    package_data=PackageData,
    package_dir = {'': 'src'},
    eager_resources = EagerResources,
    entry_points = EntryPoints,
    namespace_packages = ['ua2'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries',
    ]
)
